import std/[streams, options, parsexml]


const validCodes = [
  "AJ0", "AJC", "AJS", "AVQ", "CJS", "DT0", "DTQ", "NN0", "NN1", "NN2", 
  "PNI", "PNQ", "PNX", "VVB", "VVD", "VVG", "VVI", "VVN", "VVZ"
]


type Word = object
  code: string
  headword: string


proc isValid(headword: string): bool =
  for c in headword:
    if ord(c) > 127:
      return false # who cares about non-ascii anyway, it's hard to type
    case c:
      of '1', '2', '3', '4', '5', '6', '7', '8', '9', '0':
        return false
      of '$', '-', '\'', '/', ',', '.', '%':
        return false
      of '(', ')', '[', ']', '{', '}':
        return false
      else: 
        discard
  return true


proc getWordAttrs(x: var XmlParser): Option[Word] =
  var w: Word
  x.next()
  while x.kind != xmlElementClose and x.kind != xmlElementOpen:
    if x.kind == xmlAttribute:
      # echo "found an attribute: ", x.attrKey
      if x.attrKey == "c5":
        w.code = x.attrValue
      if x.attrKey == "hw":
        w.headword = x.attrValue

      if w.code.len > 0 and w.headword.len > 0:
        return some(w)
    x.next()


iterator iterWords*(filename: string): string =
  var file: FileStream
  try:
    file = openFileStream(filename)
  except IOError as e:
    echo "Failed to open file: ", filename
    echo "Message: ", e.msg

  var x: XmlParser
  x.open(file, filename)
  x.next()
  while x.kind != xmlEof:
    x.next()
    if x.kind == xmlElementOpen and x.elementName == "w":
      let res = x.getWordAttrs()
      if res.isSome:
        let word = res.get()
        if word.code in validCodes and isValid(word.headword):
          yield word.headword
  file.close()


when isMainModule:
  let path = r"C:\Users\Joe\Documents\Code\words\BNC\2554\download\Texts\A\A0\A00.xml"
  var count = 0
  for word in iterWords(path):
    inc count
  echo "words found: ", count
